

/*
  ---------------------------------------------------------------|
                              Optilogic                          |
                              Rover advanced                     |
                              V0.1                               |
                                                                 |
  ---------------------------------------------------------------|

*/


//IO
//HC-SR04                     Arduino
//Trig pin:                   7
//Echo pin:                   8

//Motor controller            Arduino
//pin1                        5
//pin2                        6
//pin3                        9
//Pin4                        10

#define MINSPEED 200
#define MAXSPEED 255

#define trigPin 7
#define echoPin 8

#define pin1 5
#define pin2 6
#define pin3 9
#define pin4 10

//Speed setpoint initialization
int sp = 100;

//Turning direction
bool turndir = false;
bool randomized = false;

void setup() {
  //Define pins as IO
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(pin1, OUTPUT);
  pinMode(pin2, OUTPUT);
  pinMode(pin3, OUTPUT);
  pinMode(pin4, OUTPUT);

  //Begin Serial Communication
  Serial.begin(9600);


  //Send Serial msg
  Serial.println("--------------------");
  Serial.println("Optimatic Rover V0.1");
  Serial.print("Starting...");

  //Delay motorstart 5s
  while (millis() < 5000) {
    Serial.print(".");
    delay(500);
  }
  Serial.print("\n");
  Serial.println("--------------------");
}

void loop() {
  //Get the distance
  long dist = GetDistance();
  //Serial.println(dist);

  if (dist <= 20.0) Turn();
  else Forward(dist);

  delay(50); //Delay 50ms - let physics do its job



}

void Forward(int dist) {
  //Function for driving forward

  //Randomize the turndirection for next turn
  if (not randomized) {
    (random(1, 100) > 50) ? turndir = true : turndir = false;
    //Lock turn direction randomized
    randomized = true;
  }

  //Calculate speed from distance - Ramp down when near target
  int mapped = constrain(map(dist, 20, 50, MINSPEED, MAXSPEED), MINSPEED, MAXSPEED);

  //Ramp up speed when target is larger than current
  //Set speed if target is lower than current
  (sp > mapped) ? sp = mapped : sp += 10;
  //Do not go above 255 or below 100
  sp = constrain(sp, MINSPEED, MAXSPEED);

  //Set speed to motors
  analogWrite(pin1, sp);
  digitalWrite(pin2, LOW);
  analogWrite(pin3, sp);
  digitalWrite(pin4, LOW);

  //Send serial notice
  Serial.print("Going forward: ");
  Serial.print(map(sp, 0, MAXSPEED, 0, 100));
  Serial.println("%");

}

void Turn() {
  //Function for turning

  //Wait 100ms
  Hold(100);

  //Reset forward ramp speed
  sp = MINSPEED;

  //Turn direction speeds
  int L = 0;
  int R = 0;


  //Send serial notice
  Serial.print("Turning: ");
  if (turndir) Serial.println("left");
  else Serial.println("right");

  //Turn the rover - ramp up
  for (int i = MINSPEED; i < MAXSPEED; i += 5) {
    //Which way to turn?
    (turndir) ? L = i : R = i;
    //Set the speeds
    analogWrite(pin1, L);
    analogWrite(pin2, R);
    analogWrite(pin3, R);
    analogWrite(pin4, L);
    delay(45); //Wait  45 ms, before incrementing speed

    
  }
  delay(100); //turn 100ms

  //Turn the rover - ramp down
  for (int i = MAXSPEED; i > MINSPEED; i -= 5) {
    //Which way to turn?
    (turndir) ? L = i : R = i;
    //Set the speeds
    analogWrite(pin1, L);
    analogWrite(pin2, R);
    analogWrite(pin3, R);
    analogWrite(pin4, L);
    delay(15); //Wait 15 ms, before decreasing speed

    
  }

  //Wait 500ms
  Hold(500);

  //Release randomization of turn direction
  randomized = false;

}

void Hold(int wait) {
  //function for stopping the motors, and waiting
  digitalWrite(pin1, LOW);
  digitalWrite(pin2, LOW);
  digitalWrite(pin3, LOW);
  digitalWrite(pin4, LOW);
  delay(wait);
}

int GetDistance() {
  //Function for getting the distance
  long sum = 0; //Sum for holding the distances before avg calculation
  
  //Measure the distance 10 times
  for (int i = 0 ; i < 10; i++) {

    //set trigger pin low for 5 µs
    digitalWrite(trigPin, LOW);
    delayMicroseconds(5);

    //Set trigger pin high for 10 µs
    digitalWrite(trigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPin, LOW);

    //time the echopin until it goes high
    long duration = pulseIn(echoPin, HIGH);

    //Add to sum for averaging
    sum += duration;

  }
  //Calculate the distance in cm from duration average
  //((Duration * speed of sound) / 2 distances)
  long distance = (((sum / 10) * 0.034) / 2);
  //Send serial notice
  Serial.print("Distance: ");
  Serial.println(distance);

  //Return the calculated distance
  return (distance);
  

}
